#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : diags.py
# Contents: create width detection diagrams
# Author  : Christian Borgelt
# History : 2013.05.09 file created
#           2014.05.27 modules placed in sibling directories
#-----------------------------------------------------------------------
from sys     import argv, path
from os.path import join
from math    import log10, floor
UTILDIR = join('..', 'util')
if UTILDIR not in path: path.insert(0, UTILDIR)
from id2par  import get_args, project

#-----------------------------------------------------------------------

oo = float('+inf')              # infinity as a constant

#cmax = zmax = 9
cmax = zmax = 12

#-----------------------------------------------------------------------

def header (out, pro):
    out.write('verbatimtex\n')
    out.write('\\documentclass[12pt]{article}\n')
    out.write('\\usepackage{mathptmx}\n')
    out.write('\\def\\hnorm{\\hbox to0pt{\\phantom{g)}\\hss}}\n')
    out.write('\\begin{document}\n')
    out.write('etex\n\n')
    out.write('prologues := ' +str(pro) +';\n\n')
    out.write('input axes;\n')
    out.write('input barchart;\n')
    out.write('input linechart;\n\n')
    out.write('picture PatSize; '
             +'PatSize = btex \\small pattern size~$z$\\hnorm etex;\n')
    out.write('picture Coins;   '
             +'Coins   = btex \\small coincidences~$c$\\hnorm etex;\n')
    out.write('picture LogPats; '
             +'LogPats = btex \\small '
             +'$\\log(\\mbox{\\#patterns})$\\hnorm etex;\n')
    out.write('numeric expand; expand = 1/3;\n\n')
    out.write('vardef bar_init (expr ttop, tbot) =\n')
    out.write('  numeric data[][];\n')
    out.write('  numeric zmin, zmax, n;\n')
    out.write('  zmin = -1; zmax = 3; n = 0;\n')
    out.write('  barchart.set_defaults("ptmr8r");\n')
    out.write('  barchart.clipcolor        := 0.7white;\n')
    out.write('  barchart.invalid.barcolor := 0.5white;\n')
    out.write('  barchart.invalid.max      := zmin-1.5;\n')
    out.write('  barchart.null.max         := zmin-0.5;\n')
    out.write('  barchart.set_units(30mm/'+str(cmax-1)
                                +', 30mm/'+str(zmax-1)
                                +', 24mm/(zmax-zmin));\n')
    out.write('  barchart.set_ranges(('+str(cmax-1)+',0.84,0.08), '
                                   +'('+str(zmax-1)+',0.84,0.08), '
                                   +'(zmin,zmax));\n')
    out.write('  barchart.set_view (-120, 36);\n')
    out.write('  barchart.set_light(-120, 75);\n')
    out.write('  axes.title.size       := 1.2;  % 12pt\n')
    out.write('  axes.title.skip       := 1.1 *fontsize "ptmr8r";\n')
    out.write('  axes.title.offset     := (0mm,1mm);\n')
    out.write('  axes.x.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.x.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.x.alabel.offset  := (0,4mm);\n')
    out.write('  axes.y.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.y.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.y.alabel.offset  := (0,4mm);\n')
    out.write('  axes.z.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.z.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.z.grid.thickness := 0.4pt;\n')
    out.write('  axes.z.kinked         := true;\n')
    out.write('  barchart.draw_xlabel(0, PatSize);\n')
    out.write('  barchart.draw_ylabel(0, Coins);\n')
    out.write('  axes.z.alabel.offset := (1.0mm,5.0mm);\n')
    out.write('  barchart.draw_zlabel(0, LogPats);\n')
    out.write('  barchart.draw_zticks(-1,0,1,2,3);\n')
    out.write('  barchart.draw_xticks("2"')
    for i in range(3,cmax+1):
        s = str(i) if cmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
        #out.write(', "%d"' % i)
    out.write(');\n')
    out.write('  barchart.draw_yticks("2"')
    for i in range(3,zmax+1):
        s = str(i) if cmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
        #out.write(', "%d"' % i)
    out.write(');\n')
    out.write('  barchart.draw_title(-3, -1, ttop, tbot);\n')
    out.write('  barchart.draw_frame;\n')
    out.write('enddef;\n\n')
    out.write('vardef set_bars (text list) =\n')
    out.write('  barchart.set_data(data[n], list);\n')
    out.write('  n := n+1;\n')
    out.write('enddef;\n\n')

#-----------------------------------------------------------------------

def footer (out):
    out.write('end.\n')

#-----------------------------------------------------------------------

def readfile (fname):
    with open(fname, 'r') as inp:
        c = [l.split() for l in inp]
    return dict([((int(x),int(y)),float(z)) for x,y,z in c])

#-----------------------------------------------------------------------

colors = ['black', 'blue', 'red', '0.8green', '(1,0.67,0)',
          '(0,1,1)', '(1,0,1)', '0.4white', '0.8white']

#-----------------------------------------------------------------------

def linechart (out, fid, args, widths, size=0, coin=0):
    name = 'wd_n%d_p%g_r%g_j%g_%c_%d_%d' \
         % (args['ncnt'],args['period'],args['rate'],
            args['jitter']*1000,args['target'],size,coin)
    data = []                   # create the input file name
    with open(join('data', name+'.txt'), 'r') as inp:
        for line in inp:        # read the evaluation data
            line = [float(x) for x in line[:-1].split()]
            data.append((1000*line[0],line[1:]))
    x = [w for w,r in data]
    xmin = min(x); xmax = max(x); xspan = xmax-xmin
    if xspan <= 0: xspan = 1
    xmin -= 0.025*xspan; xmax += 0.025*xspan; xspan *= 1.05
    out.write('beginfig(' +str(fid) +'); % '+name+'\n' )
    out.write('  pair data[];\n')
    out.write('  linechart.set_defaults("ptmr8r");\n')
    out.write('  linechart.x.tlabel.size   := 8/10; %  8pt\n')
    out.write('  linechart.x.alabel.size   := 1;    % 10pt\n')
    out.write('  linechart.x.alabel.offset := (0,3.0mm);\n')
    out.write('  linechart.y.tlabel.size   := 8/10; %  8pt\n')
    out.write('  linechart.y.alabel.size   := 1;    % 10pt\n')
    out.write('  linechart.y.alabel.offset := (0,5.2mm);\n')
    out.write('  linechart.set_units(%fmm, 32mm);\n' % (64/xspan))
    out.write('  linechart.set_ranges((%f,%f), (0,1));\n' % (xmin,xmax))
    out.write('  linechart.draw_xticks(1, 2, 3, 4, 5, 6);\n')
    out.write('  linechart.draw_yticks(0, 0.2, 0.4, 0.6, 0.8, 1.0);\n')
    out.write('  linechart.draw_ylabel(0, "evaluation");\n')
    out.write('  linechart.draw_xlabel(0, "width/ms");\n')
    out.write('  linechart.draw_frame;\n')
    n = len(data[0][1])
    curves = [zip(x,[r[i] for w,r in data]) for i in range(n)]
    mins   = [min(y for x,y in c) for c in curves]
    maxs   = [max(y for x,y in c) for c in curves]
    norm   = [1.0/(b-a) if b > a else 1 for a,b in zip(mins,maxs)]
    curves = [[(x,(y-m)*f) for x,y in c]
              for c,f,m in zip(curves,norm,mins)]
    for i in range(n):
        out.write('  linechart.pfill(data')
        for p in curves[i]:
            if abs(p[1]) < oo: out.write(', (%f,%f)' % p)
        out.write(');\n')
        out.write('  linechart.draw_curve(data, ')
        out.write('btex $e_%d$\\hnorm etex,' % (i+1))
        out.write(' %d, 0, %s, );\n' % (i,colors[i]))
    out.write('  linechart.draw_legend((65mm,-0.00001),')
    out.write(' btex func.\\hnorm etex);\n')
    out.write('endfig;\n\n')    # draw the diagram

#-----------------------------------------------------------------------

def barchart (out, fid, cat, args, size=0, coin=0):
    if cat in ['sf','sr']:      # read excluded signatures
        name = 'n%d_p%g_r%g_w%g_%c' \
             % (args['ncnt'],args['period'],args['rate'],
                int(floor(args['width']*10000+0.5)),args['target'])
        excl = readfile(join('data','pc_'+name+'.txt')).keys()
    else: excl = set()          # or do not exclude any signatures
    name = 'n%d_p%g_r%g_j%g_w%02d_%c_%d_%d' \
         % (args['ncnt'],args['period'],args['rate'],
            args['jitter']*1000,int(floor(args['width']*10000+0.5)),
            args['target'],size,coin)
    top  = 'btex $w = %.1f$ms\\hnorm etex' % (args['width']*1000)
    bot  = '""'                 # create the diagram title
    vals = readfile(join('data',cat+'_'+name+'.txt'))
    out.write('beginfig(' +str(fid) +'); % '+cat+'_'+name+'\n')
    out.write('  bar_init('+top+', '+bot+');\n')
    for z in range(2,zmax+1):   # traverse the group sizes (neurons)
        out.write('  set_bars(')# start the next data row
        for c in range(2,cmax+1): # traverse the numbers of coincidences
            if c > 2: out.write(', ')
            x = vals[(z,c)] if (z,c) in vals else 0
            x = log10(x) if x > 0 else -2
            if (z,c) in excl: x = -3
            out.write('%f' % (x if x <= 3 else 4))
        out.write(');\n')       # print the values/fill data matrix
    out.write('  barchart.draw_data(data, 0, 5, expand);\n')
    out.write('endfig;\n\n')    # draw the diagram

#-----------------------------------------------------------------------

if __name__ == '__main__':
    grids  = [ [ ( 'ncnt',   [  100 ] ),
                 ( 'period', [    3 ] ),
                 ( 'rate',   [   20 ] ),
                 ( 'jitter', [ 0.003] ),
                 ( 'target', [ 'c'  ] ) ] ]
    widths = [ 0.0060, 0.0058, 0.0056, 0.0054, 0.0052,
               0.0050, 0.0048, 0.0046, 0.0044, 0.0042,
               0.0040, 0.0038, 0.0036, 0.0034, 0.0032,
               0.0030, 0.0028, 0.0026, 0.0024, 0.0022,
               0.0020, 0.0018, 0.0016, 0.0014, 0.0012, 0.0010 ]
    sigs   = zip(range(5,13),range(5,13)) \
           + [(5,9),(9,5),(6,10),(10,6)]
    fid    = 0
    with open(argv[1] if len(argv) > 1 else 'width.mp', 'w') as out:
        header(out, 2)
        for i in range(get_args(grids)):
            args = get_args(grids, i)
            for z,c in sigs:
                linechart(out, fid, args, widths, z, c); fid += 1
        if len(argv) < 3:
            grids[0].append(('width', widths))
            for i in range(get_args(grids)):
                args = get_args(grids, i)
                for z,c in sigs:
                    barchart(out, fid, 'su', args, z, c); fid += 1
                    barchart(out, fid, 'sf', args, z, c); fid += 1
                    barchart(out, fid, 'sr', args, z, c); fid += 1
        footer(out)
